SystemOrganization addCategory: #'OmniBrowser-Announcements'!
SystemOrganization addCategory: #'OmniBrowser-Commands'!
SystemOrganization addCategory: #'OmniBrowser-Kernel'!
SystemOrganization addCategory: #'OmniBrowser-Nodes'!
SystemOrganization addCategory: #'OmniBrowser-Notifications'!
SystemOrganization addCategory: #'OmniBrowser-Panels'!
SystemOrganization addCategory: #'OmniBrowser-Utilities'!

Notification subclass: #OBAnnouncerRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

Notification subclass: #OBInteractionRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBInteractionRequest commentStamp: 'cwp 12/7/2004 00:13' prior: 0!
OBInteractionRequest is an abstract superclass for notifications that request some interaction with the user. It's useful for catching such notifications in an exception handler, while allowing other notifications to operate normally.!

OBInteractionRequest subclass: #OBBrowseRequest
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBBrowseRequest commentStamp: 'cwp 12/7/2004 00:10' prior: 0!
This notification is raised whenever a browser needs to be opened. The default action is to open a SystemWindow in Morphic, but it can be caught in situations where that's not appropriate. The OB test suite uses this extensively. !

!OBBrowseRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 10/17/2004 18:45'!
signal: aBrowser
	^ (self new browser: aBrowser) signal! !

!OBBrowseRequest methodsFor: 'accessing' stamp: 'cwp 10/17/2004 18:45'!
browser
	^ browser! !

!OBBrowseRequest methodsFor: 'accessing' stamp: 'cwp 10/17/2004 18:45'!
browser: aBrowser
	browser _ aBrowser! !

!OBBrowseRequest methodsFor: 'accessing' stamp: 'cwp 3/19/2007 00:03'!
defaultAction
	^ OBMorphBuilder open: browser! !

!OBBrowseRequest methodsFor: 'testing' stamp: 'cwp 10/17/2004 19:32'!
isBrowseRequest
	^ true! !

OBInteractionRequest subclass: #OBChoiceRequest
	instanceVariableNames: 'prompt labels values lines'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBChoiceRequest commentStamp: 'cwp 3/5/2004 12:26' prior: 0!
This notification is used to ask the user for to choose from a list of alternatives. Its defaultAction is to open a PopUpMenu. Test cases an intercept the notification and respond programmatically.

prompt	- A string describing the choice the user is asked to make.
labels	- A list of strings describing the alternatives.
values	- When the user chooses an alternative, the corresponding item from this list is returned!

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 7/9/2006 00:07'!
labels: anArray
	^ self
		prompt: nil
		labels: anArray
		values: anArray
		lines: #()! !

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 7/9/2006 00:08'!
labels: labelArray lines: lineArray
	^ self
		prompt: nil
		labels: labelArray
		values: labelArray
		lines: lineArray! !

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 7/9/2006 00:06'!
prompt: aString labels: labelArray values: valueArray
	^ self
		prompt: aString
		labels: labelArray
		 values: valueArray
		lines: #()! !

!OBChoiceRequest class methodsFor: 'signalling' stamp: 'cwp 7/9/2006 00:05'!
prompt: aString labels: labelArray values: valueArray lines: lineArray
	^ (self new 
		setPrompt: aString 
		labels: labelArray 
		values: valueArray 
		lines: lineArray) 
			signal! !

!OBChoiceRequest methodsFor: 'signaling' stamp: 'cwp 7/9/2006 00:09'!
defaultAction
	^ values 
		at: ((PopUpMenu labelArray: labels lines: lines) startUpWithCaption: prompt)
		ifAbsent: [nil]! !

!OBChoiceRequest methodsFor: 'accessing' stamp: 'cwp 7/8/2006 23:45'!
labels
	^ labels! !

!OBChoiceRequest methodsFor: 'accessing' stamp: 'cwp 7/9/2006 00:11'!
lines
	^ lines! !

!OBChoiceRequest methodsFor: 'accessing' stamp: 'cwp 7/8/2006 23:45'!
prompt
	^ prompt! !

!OBChoiceRequest methodsFor: 'signaling' stamp: 'cwp 10/31/2006 20:38'!
select: aString
	| index |
	index _ labels indexOf: aString.
	self resume: (values at: index ifAbsent: [nil])! !

!OBChoiceRequest methodsFor: 'initializing' stamp: 'cwp 7/9/2006 00:05'!
setPrompt: aString labels: labelArray values: valueArray lines: lineArray
	prompt _ aString.
	labels _ labelArray.
	values _ valueArray.
	lines _ lineArray.! !

!OBChoiceRequest methodsFor: 'accessing' stamp: 'cwp 7/8/2006 23:45'!
values
	^ values! !

OBInteractionRequest subclass: #OBConfirmationRequest
	instanceVariableNames: 'prompt confirm cancel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBConfirmationRequest commentStamp: 'cwp 3/5/2004 12:30' prior: 0!
This notification is used to ask the user to confirm some kind potentially dangerous operation. Its default action is to open a PopUpMenu.

iVars:

prompt		- a string describing the situation the user is asked to confirm
confirm		- a string describing the action that will be taken if the user confirms
cancel		- a string describing the action that will be taken if the user does not confirm!

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:09'!
newPrompt: prompt confirm: confirm cancel: cancel
	^ self new setPrompt: prompt confirm: confirm cancel: cancel! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:13'!
prompt: prompt
	^ self prompt: prompt confirm: 'Ok'! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:12'!
prompt: prompt confirm: confirm
	^ self prompt: prompt confirm: confirm cancel: 'Cancel'! !

!OBConfirmationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2004 12:02'!
prompt: prompt confirm: confirm cancel: cancel
	^ (self newPrompt: prompt confirm: confirm cancel: cancel) signal! !

!OBConfirmationRequest methodsFor: 'responding' stamp: 'cwp 2/28/2004 12:11'!
cancel
	self resume: false! !

!OBConfirmationRequest methodsFor: 'user interaction' stamp: 'cwp 6/4/2006 14:17'!
defaultAction
	(prompt beginsWith: 'BogusD') ifTrue: [self halt].
	^ PopUpMenu confirm: prompt trueChoice: confirm falseChoice: cancel! !

!OBConfirmationRequest methodsFor: 'responding' stamp: 'cwp 2/28/2004 12:11'!
ok
	self resume: true! !

!OBConfirmationRequest methodsFor: 'initalizing' stamp: 'cwp 2/28/2004 12:00'!
setPrompt: promptString confirm: confirmString cancel: cancelString
	prompt _ promptString.
	confirm _ confirmString.
	cancel _ cancelString! !

!OBInteractionRequest methodsFor: 'as yet unclassified' stamp: 'cwp 10/17/2004 19:31'!
isBrowseRequest
	^ false! !

OBInteractionRequest subclass: #OBTextRequest
	instanceVariableNames: 'prompt template'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Notifications'!

!OBTextRequest commentStamp: 'cwp 3/5/2004 12:35' prior: 0!
This notification is used to ask the user to supply a short piece of text. Its defaultAction is to open a FillInTheBlank.

iVars:

prompt		- a string describing the text the user is asked to supply
template	- a default reply !

!OBTextRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/2/2004 21:30'!
prompt: aString
	^ self new prompt: aString; template: ''! !

!OBTextRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 2/5/2004 20:31'!
prompt: promptString template: templateString
	^ (self new prompt: promptString; template: templateString) signal! !

!OBTextRequest methodsFor: 'user interaction' stamp: 'cwp 2/2/2004 21:27'!
defaultAction
	self resume: self requestText! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:24'!
prompt
	^ prompt! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:23'!
prompt: aString
	prompt _ aString! !

!OBTextRequest methodsFor: 'user interaction' stamp: 'cwp 2/2/2004 21:27'!
requestText
	^ FillInTheBlankMorph 
		request: prompt
		initialAnswer: template
		centerAt: Sensor cursorPoint
		inWorld: World
		onCancelReturn: nil
		acceptOnCR: true! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 21:00'!
template
	^ template! !

!OBTextRequest methodsFor: 'accessing' stamp: 'cwp 2/2/2004 20:59'!
template: aString
	template _ aString! !

!LazyListMorph methodsFor: '*omnibrowser' stamp: 'cwp 2/28/2006 18:47'!
rectForRow: index
	"return a rectangle containing the row at index"
	
	| top |
	top _ self top + (index - 1 * font height).
	^ (self left @ top) extent: self width @ font height
! !

!MessageSend methodsFor: '*omnibrowser-evaluating' stamp: 'cwp 6/24/2006 18:19'!
valueWithPossibleArgs: anArray
	^ receiver 
		perform: selector 
		withArguments: (self collectArguments: anArray)! !

!PluggableListMorph methodsFor: '*omnibrowser' stamp: 'cwp 3/14/2004 13:09'!
getListDelicately
	| lazy |
	lazy _ self listMorph.
	^ (1 to: lazy getListSize) collect: [:i | lazy item: i].
	! !

!PluggableListMorph methodsFor: '*omnibrowser' stamp: 'cwp 3/14/2004 13:47'!
getListObtrusively
	| lazy |
	lazy _ self listMorph.
	^ (1 to: lazy getListSize) collect: [:i | lazy getListItem: i].
	! !

!MenuIcons class methodsFor: '*omnibrowser-services' stamp: 'cwp 7/10/2006 22:32'!
tryIcons: anArray
	| selector |
	selector := anArray detect: [:ea | self respondsTo: ea] ifNone: [self blankIcon].
	^ MenuIcons perform: selector! !

!Behavior methodsFor: '*omnibrowser-converting' stamp: 'cwp 4/17/2006 12:16'!
asAnnouncement
	^ self new! !

Object subclass: #OBAnnouncement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

OBAnnouncement subclass: #OBAboutToChange
	instanceVariableNames: 'veto'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBAboutToChange methodsFor: 'vetos' stamp: 'cwp 4/17/2006 22:18'!
isVetoed
	^ veto notNil! !

!OBAboutToChange methodsFor: 'vetos' stamp: 'cwp 4/17/2006 19:45'!
veto
	veto _ true! !

OBAnnouncement subclass: #OBAboutToChangeSilently
	instanceVariableNames: 'veto'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBAboutToChangeSilently methodsFor: 'vetos' stamp: 'cwp 3/3/2007 22:28'!
isVetoed
	^ veto notNil! !

!OBAboutToChangeSilently methodsFor: 'vetos' stamp: 'cwp 3/3/2007 22:27'!
veto
	veto _ true! !

OBAnnouncement subclass: #OBChildrenChanged
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBChildrenChanged class methodsFor: 'as yet unclassified' stamp: 'cwp 6/5/2006 00:49'!
node: aNode
	^ self new node: aNode! !

!OBChildrenChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/5/2006 00:49'!
node
	^ node! !

!OBChildrenChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/5/2006 00:49'!
node: aNode
	node _ aNode! !

OBAnnouncement subclass: #OBCommandScan
	instanceVariableNames: 'factories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBCommandScan methodsFor: 'accessing' stamp: 'cwp 11/2/2006 00:35'!
addFactory: aCommandFactory 
	factories add: aCommandFactory! !

!OBCommandScan methodsFor: 'accessing' stamp: 'cwp 11/2/2006 00:30'!
commandsOn: aNode for: aRequestor
	^ factories collect: [:ea | ea on: aNode for: aRequestor]! !

!OBCommandScan methodsFor: 'user interface' stamp: 'cwp 10/15/2006 00:33'!
groupCommands: commands satisfying: aBlock
	| groups |
	groups _ (commands select: aBlock)
				groupBy: [:svc | svc group]
				having: [:group | group isEmpty not].
	groups _ groups collect: [:ea | ea asArray sort: [:a :b | a label <= b label]].
	^ groups asArray sort: [:a :b | a anyOne group <= b anyOne group].
	! !

!OBCommandScan methodsFor: 'initialize-release' stamp: 'cwp 11/2/2006 00:30'!
initialize
	factories _ OrderedCollection new! !

!OBCommandScan methodsFor: 'user interface' stamp: 'lr 4/25/2007 09:15'!
populateMenu: aMenu withNode: aNode forRequestor: aRequestor 
	^ self 
		populateMenu: aMenu
		withNode: aNode
		forRequestor: aRequestor
		filter: [:svc | svc isActive]! !

!OBCommandScan methodsFor: 'user interface' stamp: 'cwp 10/15/2006 01:16'!
populateMenu: aMenu withNode: aNode forRequestor: aRequestor filter: aBlock

	| commands |
	commands _ self commandsOn: aNode for: aRequestor.
	(self groupCommands: commands satisfying: aBlock) do: 
		[:group |
		aMenu items isEmpty ifFalse: [aMenu addLine]. 
		group do: [:svc | svc addItemToMenu: aMenu]]
			! !

!OBCommandScan methodsFor: 'user interface' stamp: 'cwp 10/15/2006 14:06'!
processKeystroke: aCharacter withNode: aNode for: aRequestor
	| commands |
	commands _ self commandsOn: aNode for: aRequestor. 
	(commands select: [:cmd | cmd keystroke == aCharacter]) do:
		[:cmd |
		(cmd isActive and: [cmd isEnabled])
			ifTrue: [cmd execute]]! !

OBCommandScan subclass: #OBNodeCommandScan
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

OBCommandScan subclass: #OBTextCommandScan
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

OBAnnouncement subclass: #OBDefinitionChanged
	instanceVariableNames: 'definition node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBDefinitionChanged class methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:28'!
definition: aDefinition
	^ self new
		definition: aDefinition;
		yourself! !

!OBDefinitionChanged class methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:27'!
node: aNode definition: aDefinition
	^ self new
		node: aNode;
		definition: aDefinition;
		yourself! !

!OBDefinitionChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:25'!
definition
	^ definition! !

!OBDefinitionChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:26'!
definition: aDefinition
	definition _ aDefinition! !

!OBDefinitionChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/5/2006 00:47'!
node
	^ node! !

!OBDefinitionChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/5/2006 00:47'!
node: aNode
	node _ aNode! !

OBAnnouncement subclass: #OBNodeChanged
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBNodeChanged class methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:17'!
node: aNode
	^ self new node: aNode! !

!OBNodeChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:17'!
node
	^ node! !

!OBNodeChanged methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:17'!
node: aNode
	node _ aNode! !

OBAnnouncement subclass: #OBNodeCreated
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBNodeCreated class methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 01:09'!
node: aNode
	^ self new node: aNode! !

!OBNodeCreated methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 00:53'!
node
	^ node! !

!OBNodeCreated methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 00:53'!
node: aNode
	node _ aNode! !

OBAnnouncement subclass: #OBNodeDeleted
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBNodeDeleted class methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:56'!
node: aNode
	^ self new node: aNode! !

!OBNodeDeleted methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:56'!
node
	^ node! !

!OBNodeDeleted methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 11:56'!
node: aNode
	node _ aNode! !

OBAnnouncement subclass: #OBRefreshRequired
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

OBAnnouncement subclass: #OBSelectingNode
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBSelectingNode class methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 13:58'!
node: aNode
	^ self new node: aNode! !

!OBSelectingNode methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 13:58'!
node
	^ node! !

!OBSelectingNode methodsFor: 'as yet unclassified' stamp: 'cwp 6/4/2006 13:58'!
node: aNode
	node _ aNode! !

OBAnnouncement subclass: #OBSelectionChanged
	instanceVariableNames: 'node column'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBSelectionChanged class methodsFor: 'instance creation' stamp: 'cwp 4/19/2006 00:51'!
column: aColumn 
	^ self new column: aColumn! !

!OBSelectionChanged class methodsFor: 'as yet unclassified' stamp: 'cwp 4/19/2006 00:31'!
node: aNode
	^ self new node: aNode! !

!OBSelectionChanged methodsFor: 'accessing' stamp: 'cwp 4/19/2006 00:53'!
column
	^ column! !

!OBSelectionChanged methodsFor: 'accessing' stamp: 'cwp 4/19/2006 00:52'!
column: aColumn
	column _ aColumn! !

!OBSelectionChanged methodsFor: 'accessing' stamp: 'cwp 6/4/2006 12:19'!
node
	^ column 
		ifNil: [node]
		ifNotNil: [column selectedNode]! !

!OBSelectionChanged methodsFor: 'accessing' stamp: 'cwp 6/4/2006 12:20'!
node: aNode
	node _ aNode! !

Object subclass: #OBAnnouncer
	instanceVariableNames: 'subscriptions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Announcements'!

!OBAnnouncer class methodsFor: 'instance creation' stamp: 'cwp 6/4/2006 00:55'!
current
	^ OBAnnouncerRequest signal! !

!OBAnnouncer class methodsFor: 'instance creation' stamp: 'cwp 4/17/2006 11:50'!
new
	^ self basicNew initialize! !

!OBAnnouncer methodsFor: 'subscription' stamp: 'cwp 6/5/2006 00:39'!
announce: anObject

	| ann |
	ann _ anObject asAnnouncement.
	subscriptions keysAndValuesDo:
		[:class :action |
		(ann isKindOf: class) ifTrue: [action valueWithArguments: {ann}]].
	^ ann! !

!OBAnnouncer methodsFor: 'initialize-release' stamp: 'cwp 4/17/2006 11:51'!
initialize
	subscriptions _ IdentityDictionary new.! !

!OBAnnouncer methodsFor: 'subscription' stamp: 'cwp 6/3/2006 21:21'!
observe: aClass do: aValuable 
	| actions |
	actions _ subscriptions at: aClass ifAbsent: [ActionSequence new].
	subscriptions at: aClass put: (actions copyWith: aValuable).! !

!OBAnnouncer methodsFor: 'subscription' stamp: 'cwp 6/3/2006 23:31'!
observe: aClass send: aSelector to: anObject
	self
		observe: aClass
		do: (MessageSend receiver: anObject selector: aSelector)! !

!OBAnnouncer methodsFor: 'subscription' stamp: 'cwp 6/10/2006 00:40'!
unsubscribe: anObject
	subscriptions keysAndValuesDo:
		[:class :actions |
		subscriptions at: class put: (actions reject: [:ea | ea receiver == anObject])].
	subscriptions keysAndValuesRemove: [:key :value | value isEmpty]! !

Object subclass: #OBBrowser
	instanceVariableNames: 'panels announcer cmdFactories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBBrowser commentStamp: 'cwp 1/7/2005 23:17' prior: 0!
OBBrowser is the core of OmniBrower. It's the root object for each browser and the model for the SystemWindows which display them. Its main responsibily is managing panels, particularly passing update messages between them.

On the class side, OBBrowser provides some default settings for creating and opening browsers. Subclasses can override these settings to acheive customized behavior.

iVars:

panels	- A collection of objects which manage submorphs of the browser's SystemWindow.

cVars:

MetaGraphs - A dictionary matching names to metagraphs!

!OBBrowser class methodsFor: 'configuration' stamp: 'avi 9/17/2005 01:10'!
defaultMetaNode
	self subclassResponsibility! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 12/6/2004 23:20'!
defaultRootNode
	self subclassResponsibility! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:10'!
definitionPanel
	^ OBDefinitionPanel new! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:01'!
maxPanes
	^ self paneCount! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 18:12'!
metaNode: metaNode node: rootNode 
	^ self metaNode: metaNode root: rootNode selection: nil panels: self panels! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'avi 9/17/2005 01:16'!
metaNode: metaNode root: rootNode selection: selectedNode
	^ self metaNode: metaNode root: rootNode selection: selectedNode panels: self panels! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/6/2004 21:59'!
metaNode: metaNode root: rootNode selection: selectedNode panels: panels
	| browser |
	browser _ self basicNew initialize.
	panels do: [:ea | browser addPanel: ea].
	browser setMetaNode: metaNode node: rootNode.
	selectedNode ifNotNil: [browser jumpTo: selectedNode].
	^ browser! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:00'!
minPanes
	^ self paneCount! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 12/5/2004 16:55'!
navigationPanel
	^ OBColumnPanel minPanes: self minPanes maxPanes: self maxPanes! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'avi 9/16/2005 23:25'!
new
	^ self 
		metaNode: self defaultMetaNode
		root: self defaultRootNode
		selection: nil
		panels: self panels! !

!OBBrowser class methodsFor: 'auto opening' stamp: 'cwp 12/6/2004 23:22'!
open
	^ self new open! !

!OBBrowser class methodsFor: 'auto opening' stamp: 'cwp 1/22/2007 23:21'!
openRoot: aNode
	^ (self root: aNode) open! !

!OBBrowser class methodsFor: 'auto opening' stamp: 'cwp 12/5/2004 17:03'!
openRoot: rootNode selection: selectedNode
	^ (self root: rootNode selection: selectedNode) open! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 3/11/2007 19:33'!
optionalButtonPanel
	^ Preferences optionalButtons ifTrue: [OBFixedButtonPanel new]! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:07'!
optionalButtons
	^ #()! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:01'!
paneCount
	^ 4! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 12/5/2004 15:06'!
panels
	^ {self navigationPanel. self optionalButtonPanel. self definitionPanel} 
		reject: [:ea | ea isNil]! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 15:26'!
root: aNode 
	^ self root: aNode selection: nil! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'avi 9/16/2005 23:25'!
root: rootNode selection: selectedNode
	^ self metaNode: self defaultMetaNode root: rootNode selection: selectedNode panels: self panels! !

!OBBrowser class methodsFor: 'instance creation' stamp: 'cwp 12/6/2004 22:04'!
selection: selectedNode
	^ self root: self defaultRootNode selection: selectedNode! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 11:32'!
title
	^ 'OmniBrowser'! !

!OBBrowser class methodsFor: 'configuration' stamp: 'cwp 8/31/2004 13:02'!
titleForRoot: aNode
	^ self title! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 6/4/2006 00:36'!
addPanel: aPanel
	panels add: aPanel.
	aPanel browser: self.
! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 10/4/2006 10:06'!
announce: anObject
	^ self announcer announce: anObject! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 4/17/2006 15:42'!
announcer
	^ announcer! !

!OBBrowser methodsFor: 'building' stamp: 'cwp 3/18/2007 20:46'!
buildGroup: aCollection on: aBuilder
	^ aBuilder
		panelGroup: aCollection
		with: [aCollection do: [:ea | ea buildOn: aBuilder]]! !

!OBBrowser methodsFor: 'building' stamp: 'cwp 3/18/2007 20:47'!
buildOn: aBuilder
	
	| group |
	^ aBuilder browser: self with: 
		[group _ OrderedCollection with: panels first.
		panels allButFirst do: 
			[:panel | 
			group last vResizing = #spaceFill ifTrue:
				[self buildGroup: group on: aBuilder.
				group _ OrderedCollection new].
			group add: panel].
		self buildGroup: group on: aBuilder]! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 3/11/2007 18:29'!
cmdFactories
	^ cmdFactories! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 11/20/2004 21:00'!
currentNode
	^self navigationPanel currentNode! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 11/20/2004 21:00'!
currentOrRootNode
	^self navigationPanel currentOrRootNode! !

!OBBrowser methodsFor: 'building' stamp: 'cwp 11/27/2004 22:21'!
defaultBackgroundColor
	^ Color gray veryMuchLighter! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 8/31/2004 13:04'!
defaultLabel
	^ self class titleForRoot: self root! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 6/9/2006 23:29'!
dontTranscribe
	self announcer unsubscribe: self.
	self subscribe.! !

!OBBrowser methodsFor: 'initializing' stamp: 'cwp 11/2/2006 00:26'!
initialize
	panels _ OrderedCollection new.
	announcer _ OBAnnouncer new.
	cmdFactories _ #().
	self subscribe.! !

!OBBrowser methodsFor: 'initializing' stamp: 'cwp 11/2/2006 00:26'!
initializeCommands
	cmdFactories := Array streamContents: 
					[:stream | 
					(self class allSelectors select: [:ea | ea beginsWith: 'cmd']) do: 
							[:selector | 
							| result |
							result := self perform: selector.
							result isCollection 
								ifTrue: [stream nextPutAll: result]
								ifFalse: [stream nextPut: result]]]! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 11/20/2004 21:00'!
jumpTo: aNode 
	self navigationPanel jumpTo: aNode! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 11/20/2004 21:00'!
jumpToRoot
	^self navigationPanel jumpToRoot! !

!OBBrowser methodsFor: 'callbacks' stamp: 'cwp 11/20/2004 21:05'!
labelString
	| label |
	label := self navigationPanel labelString.
	^label 
		ifNil: [self defaultLabel]
		ifNotNil: [self defaultLabel , ': ' , label]! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 12/5/2004 16:25'!
navigationPanel
	^ panels 
		detect: [:ea | ea isNavigation] 
		ifNone: [self error: 'No navigation panel configured']! !

!OBBrowser methodsFor: 'callbacks' stamp: 'cwp 4/17/2006 19:52'!
okToChange
	^ (self announcer announce: OBAboutToChange) isVetoed not! !

!OBBrowser methodsFor: 'opening' stamp: 'cwp 10/17/2004 21:31'!
open
	^ OBBrowseRequest signal: self! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 9/17/2004 00:53'!
panels
	^ panels! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 4/24/2006 15:10'!
relabel: ann
	self changed: #relabel.
! !

!OBBrowser methodsFor: 'accessing' stamp: 'cwp 6/28/2006 00:44'!
requestor
	^ OBRequestor for: self! !

!OBBrowser methodsFor: 'navigating' stamp: 'cwp 9/17/2005 17:48'!
root
	^self navigationPanel root! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 11/2/2006 00:35'!
scanNodeCommands: ann 
	(cmdFactories select: [:ea | ea takesNodes]) 
		do: [:ea | ann addFactory: ea]! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 11/2/2006 00:35'!
scanTextCommands: ann 
	(cmdFactories select: [:ea | ea takesText]) do: [:ea | ann addFactory: ea]! !

!OBBrowser methodsFor: 'accessing' stamp: 'dvf 9/5/2005 17:30'!
selectionPath
	^self navigationPanel selectionPath! !

!OBBrowser methodsFor: 'initializing' stamp: 'cwp 11/1/2006 22:24'!
setMetaNode: aMetaNode node: aNode 
	self navigationPanel setMetaNode: aMetaNode node: aNode.
	self initializeCommands! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 6/4/2006 18:50'!
signalRefresh
	self announcer announce: OBRefreshRequired! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 11/1/2006 23:46'!
subscribe
	(self announcer)
		observe: OBSelectionChanged
			send: #relabel:
			to: self;
		observe: OBNodeCommandScan
			send: #scanNodeCommands:
			to: self;
		observe: OBTextCommandScan
			send: #scanTextCommands:
			to: self! !

!OBBrowser methodsFor: 'updating' stamp: 'cwp 6/6/2006 00:26'!
transcribe
	self announcer observe: OBAnnouncement do: [:ann | Transcript cr; show: ann].! !

Object subclass: #OBBuilder
	instanceVariableNames: 'current'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBBuilder class methodsFor: 'as yet unclassified' stamp: 'cwp 3/18/2007 20:47'!
build: aModel
	| builder |
	builder _ self new.
	^ aModel buildOn: builder.
	! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/13/2007 21:25'!
browser: aBrowser with: aBlock
	self subclassResponsibility! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/13/2007 21:25'!
column: aColumn with: aBlock
	self subclassResponsibility! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/13/2007 21:26'!
columnPanel: aPanel with: aBlock
	self subclassResponsibility! !

!OBBuilder methodsFor: 'accessing' stamp: 'cwp 3/16/2007 23:24'!
current: aWidget
	current _ aWidget! !

!OBBuilder methodsFor: 'accessing' stamp: 'cwp 3/16/2007 23:19'!
current: anObject do: aBlock

	| saved |
	[saved _ current.
	current _ anObject.
	aBlock value]
		ensure: [current _ saved].
	^ anObject! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/13/2007 21:26'!
definitionPanel: aPanel with: aBlock
	self subclassResponsibility! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/18/2007 22:20'!
fixedButtonPanel: aPanel with: aBlock
	self subclassResponsibility ! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/13/2007 21:27'!
modalFilter: aFilter with: aBlock
	self subclassResponsibility! !

!OBBuilder methodsFor: 'building' stamp: 'cwp 3/18/2007 20:08'!
panelGroup: aCollection with: aBlock
	self subclassResponsibility! !

!OBBuilder methodsFor: 'accessing' stamp: 'cwp 3/13/2007 21:23'!
root
	^ current! !

Object subclass: #OBColumn
	instanceVariableNames: 'panel filter children selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBColumn commentStamp: 'cwp 1/7/2005 23:19' prior: 0!
An OBColumn manages a list of nodes, which it displays in a PluggableListMorph in the pane scroller at the top of the browser. All instances of OBColumn belong to an OBColumnPanel. It's main responsibility is keeping its list - and those of its neighbours - up to date. Each column has a MetaNode, which provides the list contents. It uses a filter to meditate between its self and the MetaNode.

iVars:

panel		- the panel which owns the column
filter		- the filter which manages the column's MetaNode.
parent		- the node selected in the column to the left of this column
children 	- the nodes which make up this column's list
selection 	- the index of the node selected by the user!

!OBColumn class methodsFor: 'instance creation' stamp: 'cwp 9/18/2005 13:26'!
inPanel: aBrowser
	^ self new setPanel: aBrowser! !

!OBColumn class methodsFor: 'instance creation' stamp: 'cwp 9/18/2005 13:25'!
inPanel: aPanel metaNode: aMetaNode node: aNode
	^ self new
			setPanel: aPanel
			metaNode: aMetaNode
			node: aNode! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 6/4/2006 00:37'!
acceptDroppingMorph: transferMorph event: evt inMorph: listMorph
	^ [(self nodeForDroppedMorph: transferMorph event: evt inMorph: listMorph)
		acceptDroppedNode: transferMorph passenger]
			on: OBAnnouncerRequest
			do: [:notification | notification resume: self announcer]
! !

!OBColumn methodsFor: 'commands' stamp: 'cwp 11/1/2006 23:15'!
addCommandsToMenu: aMenu 
	| scan |
	scan := self announce: OBNodeCommandScan.
	scan 
		populateMenu: aMenu
		withNode: self parent
		forRequestor: self.
	self hasSelection 
		ifTrue: 
			[scan 
				populateMenu: aMenu
				withNode: self selectedNode
				forRequestor: self]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 10/13/2006 09:05'!
announce: anAnnouncement 
	^ self browser announce: anAnnouncement! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 4/17/2006 15:52'!
announcer
	^ panel announcer! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:49'!
basicNext
	^panel columnAfter: self! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 11/17/2004 00:18'!
browser
	^ panel browser! !

!OBColumn methodsFor: 'building' stamp: 'cwp 3/16/2007 23:09'!
buildOn: aBuilder
	^ aBuilder 
		column: self 
		with: [filter ifNotNil: [filter buildOn: aBuilder]]! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 12/14/2003 22:15'!
buttonHeight
	^ 20! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 6/9/2006 22:35'!
childrenChanged: announcement
	(self parent = announcement node) ifTrue:
		[self refresh]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 9/25/2005 23:56'!
clear
	filter _ nil.
	children _ #().
	self clearSelection.
	self changed: #list.
	self changed: #selection.
	self changed: #filter.! !

!OBColumn methodsFor: 'selecting' stamp: 'avi 3/6/2004 18:05'!
clearSelection
	selection _ 0! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 2/9/2004 21:06'!
columnWithHeader
	| col bh |
	col _ BorderedMorph new
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		clipSubmorphs: true;
		color: Color transparent;
		cellInset: 0;
		borderWidth: 0;
		layoutPolicy: ProportionalLayout new.
	
	bh _ self buttonHeight negated.
	col
		addMorph: self listMorph
		fullFrame: (LayoutFrame fractions: (0@0 corner: 1@1) offsets: (0@0 corner: 0@bh)).
	col
		addMorph: self filter buttonMorph
		fullFrame: (LayoutFrame fractions: (0@1 corner: 1@1) offsets: (0@bh corner: 0@0)).
		

	^ col! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 9/24/2005 22:56'!
createNext
	| nextMetaNode created |
	nextMetaNode := self nextMetaNode.
	created := nextMetaNode columnInPanel: panel node: self selectedNode.
	panel pushColumn: created.
	^created! !

!OBColumn methodsFor: 'printing' stamp: 'avi 3/6/2004 16:23'!
descriptor
	self isEmpty ifTrue: [^ 'empty'].
	self hasSelection ifTrue: [^ self selectedNode name].
	^ ''! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 13:45'!
dragEnabled
	^ true! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:35'!
dragPassengerFor: item inMorph: listMorph
	^ self nodeForItem: item inMorph: listMorph! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:37'!
dragTransferType
	^ #OmniBrowser! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:36'!
dragTransferTypeForMorph: listMorph
	^ self dragTransferType! !

!OBColumn methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:30'!
dropEnabled
	^ true! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/9/2004 20:58'!
filter
	^ filter! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/12/2004 18:47'!
filter: aFilter
	filter _ aFilter monitor: self.
	self changed: #filter.! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 9/24/2005 22:29'!
getChildren
	children _ self filter nodesForParent: self parent.
	^ children! !

!OBColumn methodsFor: 'testing' stamp: 'avi 3/6/2004 23:07'!
hasSelection
	^ self selection > 0! !

!OBColumn methodsFor: 'testing' stamp: 'cwp 12/20/2004 22:09'!
includesNode: aNode
	^ children includes: aNode! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/28/2006 08:27'!
isEmpty
	^ children isEmpty! !

!OBColumn methodsFor: 'testing' stamp: 'cwp 10/9/2006 15:41'!
isSelected: aNode
	^ self selectedNode == aNode! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 3/13/2007 00:52'!
keystroke: aChar from: aMorph
	| scan |
	self isEmpty ifTrue: [^ self].	
	scan _ self announcer announce: OBNodeCommandScan.
	scan processKeystroke: aChar withNode: self parent for: self.
	self hasSelection ifTrue: [scan 
								processKeystroke: aChar 
								withNode: self selectedNode 
								for: self].! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 12/8/2004 21:21'!
list
	^ children 
		ifNil: [#()]
		ifNotNil: [children collect: [:ea | ea displayString]]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 12/8/2004 21:22'!
listAt: index
	^ (children at: index ifAbsent: [^ '']) displayString! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 6/9/2006 23:17'!
listChanged
	self refreshAndSignal: true! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 2/28/2006 08:37'!
listMorph
	^ (PluggableListMorph
		on: self
		list: #list
		selected: #selection
		changeSelected: #selection:
		menu: #menu:
		keystroke: #keystroke:from:)
			getListElementSelector: #listAt:;
			getListSizeSelector: #listSize;
			dragEnabled: self dragEnabled;
			dropEnabled: self dropEnabled;
			borderWidth: 0;
			autoDeselect: false;
			adoptPaneColor: panel defaultBackgroundColor;
			yourself! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 2/29/2004 21:48'!
listSize
	^ children
		ifNil: [0]
		ifNotNil: [children size]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 3/13/2007 00:51'!
menu: aMenu 
	self isEmpty ifFalse: [self addCommandsToMenu: aMenu].
	^aMenu! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/9/2004 20:59'!
metaNode
	^ self filter metaNode! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 2/12/2004 20:50'!
next
	^ self basicNext
		ifNil: [(self hasSelection and: [self shouldBeLast not])
				ifTrue: [self createNext]]
	! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 3/3/2004 00:11'!
nextMetaNode
	^ self selectedNode metaNode! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 6/9/2006 23:19'!
nodeChanged: ann
	(children includes: ann node) ifTrue: [self changed: #list]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 6/4/2006 12:03'!
nodeDeleted: announcement 
	"This gets called if an action causes the currently selected node to be deleted."

	self selectedNode = announcement node
		ifTrue: 
			[self getChildren.
			self changed: #list.
			self selection: 0]! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 3/3/2004 02:23'!
nodeForDroppedMorph: transferMorph event: evt inMorph: pluggableListMorph
	| index item |
	index _ pluggableListMorph rowAtLocation: evt position.
	index = 0 ifTrue: [^ nil].
	item _ pluggableListMorph listMorph item: index.
	^ self nodeForItem: item inMorph: pluggableListMorph! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 6/10/2006 21:59'!
nodeForItem: item inMorph: pluggableListMorph
	^ children 
		detect: [:child | child displayString = item contents asString]
		ifNone: [item contents]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 4/18/2006 00:25'!
okToChange
	^ (self announcer announce: OBAboutToChange) isVetoed not! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 2/12/2004 18:11'!
paneWithHeader
	^ OBPane forColumn: self withFilter: self filter! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 9/24/2005 22:45'!
parent
	^ panel parentNodeForColumn: self! !

!OBColumn methodsFor: 'accessing' stamp: 'cwp 6/8/2006 01:32'!
parent: aNode
	
	self filter: aNode metaNode filter.
	self getChildren.
	self clearSelection.
	self changed: #list.
	panel clearAfter: self! !

!OBColumn methodsFor: 'printing' stamp: 'cwp 2/11/2004 23:31'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self descriptor.
	aStream nextPut: $)! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 6/9/2006 23:08'!
refresh
	self refreshAndSignal: false! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 6/9/2006 23:08'!
refresh: ann
	self refreshAndSignal: false! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 6/9/2006 23:08'!
refreshAndSignal: aBoolean
	| node oldChildren shouldSignal |
	shouldSignal _ aBoolean.
	self isEmpty ifTrue: [^self].
	node := self selectedNode.
	oldChildren := children.
	self getChildren.
	children = oldChildren ifFalse: 
		[self selectSilently: node.
		self hasSelection ifFalse: 
			[shouldSignal _ true]].
	shouldSignal ifTrue: [self signalSelectionChanged].
	self changed: #list! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 12/8/2004 22:10'!
select: aNode
	self selection: (children indexOf: (children detect: [:ea | ea = aNode] ifNone: []))! !

!OBColumn methodsFor: 'nodes' stamp: 'cwp 9/26/2005 00:14'!
selectAncestorOf: aNode
	| ancestor |
	ancestor _ self filter selectAncestorOf: aNode withParent: self parent.
	ancestor ifNotNil:
		[self 
			getChildren; 
			changed: #list;
			select: ancestor.
		panel selected: self].
	^ ancestor! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 12/20/2004 22:54'!
selectSilently: aNode
	selection _ children indexOf: (children 
										detect: [:ea | ea name = aNode name] 
										ifNone: [nil])! !

!OBColumn methodsFor: 'selecting' stamp: 'cwp 2/2/2004 21:54'!
selectedNode
	^ children at: self selection ifAbsent: [].! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 11/5/2003 19:01'!
selection
	^ selection ifNil: [0]! !

!OBColumn methodsFor: 'callbacks' stamp: 'cwp 6/5/2006 23:37'!
selection: anInteger
	selection _ anInteger.
	self signalSelectionChanged.
	self changed: #selection.
! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 6/8/2006 00:27'!
selectionChanged: ann
	! !

!OBColumn methodsFor: 'initializing' stamp: 'cwp 6/3/2006 23:38'!
setPanel: aPanel 
	panel := aPanel.
	self subscribe.
	children := #().
	self clearSelection! !

!OBColumn methodsFor: 'initializing' stamp: 'cwp 6/3/2006 23:39'!
setPanel: aPanel metaNode: aMetaNode node: aNode 
	panel := aPanel.
	self subscribe.
	filter := aMetaNode filter monitor: self.
	children _ self filter nodesForParent: aNode.
	self clearSelection! !

!OBColumn methodsFor: 'testing' stamp: 'avi 3/6/2004 16:27'!
shouldBeLast
	^ self hasSelection not or: [self nextMetaNode hasChildren not]! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 6/5/2006 23:31'!
signalSelectionChanged
	self announcer announce: (OBSelectionChanged column: self)! !

!OBColumn methodsFor: 'morphic' stamp: 'cwp 2/12/2004 18:10'!
simplePane
	^ OBPane forColumn: self! !

!OBColumn methodsFor: 'updating' stamp: 'cwp 6/10/2006 00:18'!
subscribe
	self announcer 
		observe: OBRefreshRequired send: #refresh: to: self;
		observe: OBNodeChanged send: #nodeChanged: to: self;
		observe: OBNodeDeleted send: #nodeDeleted: to: self;
		observe: OBChildrenChanged send: #childrenChanged: to: self;
		observe: OBSelectionChanged send: #selectionChanged: to: self.! !

!OBColumn methodsFor: 'testing' stamp: 'cwp 2/12/2004 20:18'!
wantsButton
	^ self filter notNil and: [self filter wantsButton]! !

!OBColumn methodsFor: 'drag and drop' stamp: 'cwp 2/25/2004 21:26'!
wantsDroppedMorph: transferMorph event: evt inMorph: listMorph
	| node |
	(transferMorph isKindOf: TransferMorph) ifFalse: [^ false].
	transferMorph dragTransferType == self dragTransferType ifFalse: [^ false].
	node _ self nodeForDroppedMorph: transferMorph event: evt inMorph: listMorph.
	^ node notNil and: [node wantsDroppedNode: transferMorph passenger]! !

Object subclass: #OBCommand
	instanceVariableNames: 'target requestor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Commands'!

!OBCommand class methodsFor: 'as yet unclassified' stamp: 'cwp 10/14/2006 23:29'!
on: target for: requestor
	^ self new setTarget: target requestor: requestor! !

!OBCommand class methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 00:58'!
takesNodes
	^ true! !

!OBCommand class methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 00:58'!
takesText
	^ false! !

!OBCommand methodsFor: 'user interface' stamp: 'cwp 10/15/2006 01:15'!
addItemToMenu: aMenu
	self isActive ifTrue:
		[aMenu
			add: self labelWithKeystroke
			target: self 
			selector: #execute.
		aMenu lastItem isEnabled: self isEnabled.
		Preferences menuWithIcons & self icon notNil
			ifTrue: [aMenu lastItem icon: self icon].
		aMenu addBlankIconsIfNecessary: MenuIcons blankIcon]! !

!OBCommand methodsFor: 'accessing' stamp: 'lr 4/25/2007 09:27'!
buttonLabel
	^ self label! !

!OBCommand methodsFor: 'execution' stamp: 'cwp 10/15/2006 14:12'!
execute
	! !

!OBCommand methodsFor: 'accessing' stamp: 'cwp 10/6/2006 21:44'!
group
	^ #general! !

!OBCommand methodsFor: 'accessing' stamp: 'cwp 10/6/2006 18:45'!
icon
	^ nil! !

!OBCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 22:05'!
isActive
	^ false! !

!OBCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 19:31'!
isEnabled
	^ true! !

!OBCommand methodsFor: 'accessing' stamp: 'cwp 9/27/2006 22:53'!
keystroke
	^ nil
! !

!OBCommand methodsFor: 'accessing' stamp: 'cwp 9/27/2006 22:53'!
label
	self subclassResponsibility
! !

!OBCommand methodsFor: 'user interface' stamp: 'lr 4/25/2007 09:28'!
labelMorph
	^ (StringMorph 
		contents: self buttonLabel 
		font: TextStyle defaultFont)
			color: ((self isActive and: [self isEnabled]) ifTrue: [Color black] ifFalse: [Color gray]);
			yourself! !

!OBCommand methodsFor: 'accessing' stamp: 'cwp 10/6/2006 21:19'!
labelWithKeystroke
	^ self keystroke
		ifNil: [self label]
		ifNotNil: [self label, ' (', self keystroke asString, ')']! !

!OBCommand methodsFor: 'accessing' stamp: 'cwp 10/14/2006 23:39'!
on: anObject for: aRequestor
	^ self class on: anObject for: aRequestor! !

!OBCommand methodsFor: 'user interface' stamp: 'cwp 10/15/2006 12:37'!
perform: aSelector orSendTo: anObject
	self perform: aSelector! !

!OBCommand methodsFor: 'user interface' stamp: 'cwp 10/9/2006 13:49'!
select: aNode with: anAnnouncer
	(anAnnouncer announce: OBAboutToChange)
		isVetoed ifFalse: [anAnnouncer announce: (OBSelectingNode node: aNode)]! !

!OBCommand methodsFor: 'initialize-release' stamp: 'cwp 10/14/2006 23:31'!
setTarget: anObject requestor: aRequestor
	target _ anObject.
	requestor _ aRequestor! !

!OBCommand methodsFor: 'testing' stamp: 'cwp 10/14/2006 19:38'!
takesNodes
	^ true! !

!OBCommand methodsFor: 'testing' stamp: 'cwp 10/14/2006 19:37'!
takesText
	^ false! !

!OBCommand methodsFor: 'testing' stamp: 'cwp 9/27/2006 22:53'!
wantsButton
	"Put this here for compatibility. Eventually this will be obsolete"

	^ false
! !

OBCommand subclass: #OBPluggableCommand
	instanceVariableNames: 'action active enabled label keystroke icon buttonLabel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Commands'!

!OBPluggableCommand class methodsFor: 'as yet unclassified' stamp: 'cwp 6/18/2006 18:53'!
action: aValuable
	^ self new action: aValuable! !

!OBPluggableCommand class methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 13:50'!
action: aValuable active: aValuable2
	^ self new 
		action: aValuable;
		active: aValuable2;
		yourself! !

!OBPluggableCommand class methodsFor: 'as yet unclassified' stamp: 'cwp 6/24/2006 15:43'!
new
	^ self basicNew initialize! !

!OBPluggableCommand class methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 00:57'!
takesNodes
	^ true! !

!OBPluggableCommand class methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 00:58'!
takesText
	^ false! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/15/2006 13:05'!
action: aValuable
	action _ aValuable! !

!OBPluggableCommand methodsFor: 'as yet unclassified' stamp: 'cwp 10/15/2006 13:07'!
active: aValuable
	active _ aValuable! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/15/2006 13:41'!
buttonLabel
	^ buttonLabel! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/15/2006 13:40'!
buttonLabel: aString
	buttonLabel _ aString! !

!OBPluggableCommand methodsFor: 'morphic' stamp: 'cwp 6/24/2006 19:41'!
buttonLabelMorph
	^ StringMorph 
		contents: self buttonLabel 
		font: Preferences standardButtonFont! !

!OBPluggableCommand methodsFor: 'morphic' stamp: 'cwp 6/24/2006 19:41'!
buttonMorph
	^ (PluggableButtonMorph
		on: self
		getState: nil
		action: #trigger
		label: #buttonLabelMorph)	
			onColor: Color lightGray lighter offColor: Color lightGray twiceLighter;
			borderWidth: 2;
			borderRaised;
			hResizing: #spaceFill;
			vResizing: #spaceFill;
			setBalloonText: label! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/15/2006 13:31'!
enabled: aBlock
	enabled _ aBlock! !

!OBPluggableCommand methodsFor: 'execute' stamp: 'cwp 10/15/2006 13:08'!
execute
	^ action valueWithPossibleArgs: {target. requestor}! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/15/2006 12:53'!
execute: aBlock
	execute _ aBlock! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/6/2006 21:46'!
group
	^ #general! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/15/2006 13:37'!
icon
	^ icon! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 10/15/2006 13:37'!
icon: anIcon
	icon _ anIcon! !

!OBPluggableCommand methodsFor: 'initialize-release' stamp: 'cwp 11/2/2006 00:39'!
initialize
	action _ [].
	active _ [true].
	enabled _ [true].
	label _ 'a command'.! !

!OBPluggableCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 13:16'!
isActive
	^ active valueWithPossibleArgs: {target. requestor}! !

!OBPluggableCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 13:17'!
isEnabled
	^ enabled valueWithPossibleArgs: {target. requestor}! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 6/24/2006 19:04'!
keystroke
	^ keystroke! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 6/24/2006 19:04'!
keystroke: aCharacter
	keystroke _ aCharacter! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 6/24/2006 19:04'!
label
	^ label! !

!OBPluggableCommand methodsFor: 'accessing' stamp: 'cwp 6/24/2006 18:58'!
label: aString
	label _ aString! !

!OBPluggableCommand methodsFor: 'morphic' stamp: 'cwp 6/24/2006 19:05'!
labelWithKeystroke
	^keystroke
		ifNil: [label]
		ifNotNil: [label, ' (', keystroke asString, ')']! !

!OBPluggableCommand methodsFor: 'converting' stamp: 'cwp 10/15/2006 12:46'!
on: aNode for: aRequestor 
	| inst |
	inst _ self copy.
	inst setTarget: aNode requestor: aRequestor.
	^ inst! !

!OBPluggableCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 00:43'!
takesNodes
	^ true! !

!OBPluggableCommand methodsFor: 'testing' stamp: 'cwp 6/25/2006 00:43'!
useLineAfter
	^ false! !

!OBPluggableCommand methodsFor: 'testing' stamp: 'cwp 6/24/2006 19:46'!
wantsButton
	^ self buttonLabel notNil! !

Object subclass: #OBDefinition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBDefinition commentStamp: 'cwp 1/7/2005 23:38' prior: 0!
The responsibility of a definition is to express a node's composition textually and respond to changes in the text by updating the node it represents. For example,a file browser might use a file definition to allow editing of a files contents.!

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:51'!
accept: aText notifying: aController
	^ self text: aText! !

!OBDefinition methodsFor: 'updating' stamp: 'cwp 7/25/2006 18:26'!
asAnnouncement
	^ OBDefinitionChanged definition: self! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:08'!
doItContext
	^ nil! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 2/28/2005 23:23'!
doItReceiver
	| class |
	class _ self selectedClass.
	^ class
		ifNotNil: [class theNonMetaClass]
		ifNil: [FakeClassPool new]! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 22:16'!
selection
	^ 1 to: 0! !

!OBDefinition methodsFor: 'updating' stamp: 'cwp 6/4/2006 18:53'!
signalChange
	OBAnnouncer current announce: (OBDefinitionChanged definition: self)! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:52'!
text
	^ ''! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:52'!
text: aText
	^ false! !

!OBDefinition methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 22:16'!
textSelection
	^ self selection! !

Object subclass: #OBFilter
	instanceVariableNames: 'metaNode monitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBFilter commentStamp: 'cwp 3/4/2004 21:53' prior: 0!
A browser's metagraph defines the way in which the user may traverse the graph of objects which make up the browser's domain. But it's not always desirable to have all the nodes made available by the metagraph to be visible in the browser. An filter provides a strategy for filtering out some of the nodes from the display. 

OBFilter provides a "null" filter - one that does no filtering at all - and serves as a superclass for other filters. !

!OBFilter class methodsFor: 'as yet unclassified' stamp: 'cwp 2/9/2004 20:54'!
forMetaNode: aMetaNode
	^ self new setMetaNode: aMetaNode! !

!OBFilter methodsFor: 'user interface' stamp: 'cwp 10/24/2006 10:18'!
buildOn: aBuilder
	! !

!OBFilter methodsFor: 'public' stamp: 'cwp 2/9/2004 20:50'!
children
	^ metaNode children! !

!OBFilter methodsFor: 'accessing' stamp: 'cwp 2/11/2004 23:48'!
metaNode
	^ metaNode! !

!OBFilter methodsFor: 'accessing' stamp: 'cwp 2/9/2004 20:56'!
monitor: aMonitor
	monitor _ aMonitor! !

!OBFilter methodsFor: 'public' stamp: 'cwp 2/9/2004 20:52'!
nodesForParent: aNode
	^ metaNode nodesForParent: aNode! !

!OBFilter methodsFor: 'public' stamp: 'cwp 2/29/2004 14:34'!
selectAncestorOf: aNode withParent: parentNode
	^ (metaNode nodesForParent: parentNode)
		detect: [:child | child isAncestorOf: aNode] 
		ifNone: [nil]! !

!OBFilter methodsFor: 'initalizing' stamp: 'cwp 2/9/2004 20:54'!
setMetaNode: aMetaNode
	metaNode _ aMetaNode! !

!OBFilter methodsFor: 'queries' stamp: 'cwp 2/9/2004 21:06'!
wantsButton
	^ false! !

OBFilter subclass: #OBModalFilter
	instanceVariableNames: 'selection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBModalFilter commentStamp: 'cwp 1/7/2005 23:39' prior: 0!
OBModalFilter is used to implement the 'instance/?/class' buttons in a standard class browser. In functional terms it filters the nodes of a column according to the edge of the metagraph that they correspond to. OBModalFilter displays an OBRadioButtonBar in its column's pane, with one button per edge. Only nodes from the currently selected edge are allowed in the column.

iVars:

selection	- the currently selected edge!

!OBModalFilter methodsFor: 'user interface' stamp: 'cwp 3/16/2007 23:09'!
buildOn: aBuilder
	^ aBuilder modalFilter: self with: []! !

!OBModalFilter methodsFor: 'morphs' stamp: 'cwp 11/27/2004 18:01'!
buttonMorph
	^ OBRadioButtonBar
		on: self
		list: #list
		selected: #selection
		changeSelected: #selection:! !

!OBModalFilter methodsFor: 'selection' stamp: 'cwp 2/9/2004 21:11'!
incrementSelection
	selection _ selection + 1.
	(selection > metaNode edges size) ifTrue: [selection _ 1].! !

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 2/25/2004 00:28'!
list
	^ metaNode edges collect: [:edge | edge label]! !

!OBModalFilter methodsFor: 'public' stamp: 'cwp 3/3/2004 00:23'!
nodesForParent: aNode
	^ self selectedEdge nodesForParent: aNode! !

!OBModalFilter methodsFor: 'public' stamp: 'cwp 2/29/2004 14:35'!
selectAncestorOf: aNode withParent: parentNode
	| ancestor |
	1 	to: metaNode edges size 
	  	do: [:i | selection _ i.
			ancestor _ (self nodesForParent: parentNode)
				detect: [:child | child isAncestorOf: aNode] 
				ifNone: [nil].
			ancestor ifNotNil: [self changed: #selection. ^ ancestor]].
	^ nil! !

!OBModalFilter methodsFor: 'selection' stamp: 'cwp 2/9/2004 21:11'!
selectedEdge
	^ metaNode edges at: self selection ! !

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 2/9/2004 21:10'!
selection
	^ selection ifNil: [selection _ 1]! !

!OBModalFilter methodsFor: 'callbacks' stamp: 'cwp 6/6/2006 00:20'!
selection: anInteger
	selection _ anInteger.
	self changed: #selection.
	monitor listChanged.
! !

!OBModalFilter methodsFor: 'queries' stamp: 'cwp 2/9/2004 21:08'!
wantsButton
	^ true! !

Object subclass: #OBMetaEdge
	instanceVariableNames: 'label selector metaNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBMetaEdge commentStamp: 'cwp 1/7/2005 23:20' prior: 0!
An OBMetaEdge is an edge in the browser's metagraph. It represents a message sent to a node to obtain further nodes. It is refered to by the "parent" metanode, and refers to the "child" metanode.

iVars:

label 		-  a string describing the metaNode, for filters which allow the user 
			   to choose which edges to follow
selector	- when a node is selected by the user, this message will be 
			  sent to it to obtain its children
metaNode 	- a MetaNode corresponding to the nodes answered by the above message
		!

!OBMetaEdge class methodsFor: 'as yet unclassified' stamp: 'cwp 2/7/2004 22:35'!
label: aString selector: aSelector metaNode: aMetaNode
	^ self new setLabel: aString selector: aSelector metaNode: aMetaNode! !

!OBMetaEdge class methodsFor: 'as yet unclassified' stamp: 'cwp 2/9/2004 22:32'!
selector: aSelector metaNode: aMetaNode
	^ self new setLabel: aSelector asString selector: aSelector metaNode: aMetaNode! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 2/7/2004 22:40'!
label
	^ label! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 2/7/2004 22:40'!
metaNode
	^ metaNode! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 3/3/2004 00:12'!
nodesForParent: aNode
	^ (aNode perform: selector) do: [:ea | ea metaNode: metaNode]
! !

!OBMetaEdge methodsFor: 'printing' stamp: 'dvf 8/16/2005 10:03'!
printOn: aStream
	aStream nextPutAll: selector printString, '->'. 
	metaNode shortPrintOn: aStream.! !

!OBMetaEdge methodsFor: 'accessing' stamp: 'cwp 2/7/2004 22:40'!
selector
	^ selector! !

!OBMetaEdge methodsFor: 'initializing' stamp: 'cwp 2/7/2004 22:36'!
setLabel: aString selector: aSelector metaNode: aMetaNode
	label _ aString.
	selector _ aSelector.
	metaNode _ aMetaNode! !

Object subclass: #OBMetaNode
	instanceVariableNames: 'name filterClass columnClass edges displaySelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBMetaNode commentStamp: 'cwp 1/7/2005 23:23' prior: 0!
A MetaNode represents a hypothetical node in the browser's domain graph.

iVars:

filterClass			- the class used to filter this hypothetical node's children
columnClass		- the class of column used to display this node's children
edges				- messages that could be sent to the node to obtain children
actors				- providers of functionality for manipulating the node
displaySelector		- the message used to retreive a nodes display name!

!OBMetaNode class methodsFor: 'instance creation' stamp: 'cwp 3/6/2004 21:06'!
named: aString
	^ self new name: aString! !

!OBMetaNode class methodsFor: 'instance creation' stamp: 'cwp 3/3/2004 23:08'!
new
	^ self basicNew initialize! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/9/2004 20:28'!
childAt: aSelector labeled: aString put: aMetaNode
	edges add: (OBMetaEdge label: aString selector: aSelector metaNode: aMetaNode)! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:35'!
childAt: aSelector put: aMetaNode
	edges add: (OBMetaEdge selector: aSelector metaNode: aMetaNode)! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:44'!
children
	^ edges collect: [:edge | edge metaNode]! !

!OBMetaNode methodsFor: 'columns' stamp: 'avi 3/4/2004 03:11'!
columnClass: aClass
	columnClass _ aClass! !

!OBMetaNode methodsFor: 'columns' stamp: 'cwp 11/16/2004 22:03'!
columnInPanel: aBrowser node: aNode
	columnClass ifNil: [columnClass _ OBColumn].
	^ columnClass
		inPanel: aBrowser
		metaNode: self
		node: aNode! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 12/8/2004 21:41'!
displaySelector
	^ displaySelector ifNil: [displaySelector _ #name]! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 12/8/2004 21:42'!
displaySelector: aSelector
	displaySelector _ aSelector! !

!OBMetaNode methodsFor: 'metagraph' stamp: 'cwp 2/9/2004 21:12'!
edges
	^ edges! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 2/9/2004 21:26'!
filter
	filterClass ifNil: [filterClass _ OBFilter].
	^ filterClass forMetaNode: self! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 2/9/2004 21:02'!
filterClass: aClass
	filterClass _ aClass! !

!OBMetaNode methodsFor: 'children' stamp: 'cwp 2/7/2004 22:38'!
hasChildren
	^ edges isEmpty not! !

!OBMetaNode methodsFor: 'initializing' stamp: 'cwp 3/13/2007 00:55'!
initialize
	edges _ OrderedCollection new.
! !

!OBMetaNode methodsFor: 'filtering' stamp: 'cwp 2/9/2004 21:00'!
metaNode
	^ self! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 3/6/2004 21:54'!
name
	^ name ifNil: ['MetaNode']! !

!OBMetaNode methodsFor: 'accessing' stamp: 'cwp 3/6/2004 21:05'!
name: aString
	name _ aString! !

!OBMetaNode methodsFor: 'nodes' stamp: 'cwp 3/3/2004 00:22'!
nodesForParent: aNode
	^ edges gather: [:edge | edge nodesForParent: aNode]! !

!OBMetaNode methodsFor: 'printing' stamp: 'dvf 8/16/2005 10:01'!
printOn: aStream
	aStream nextPutAll: self name;cr.
	edges do: [:e | e printOn: aStream. aStream cr].! !

!OBMetaNode methodsFor: 'printing' stamp: 'dvf 8/16/2005 10:01'!
shortPrintOn: aStream
	aStream nextPutAll: self name.! !

Object subclass: #OBNode
	instanceVariableNames: 'metaNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBNode commentStamp: 'cwp 3/4/2004 22:20' prior: 0!
A node is a wrapper for an object in the browser's domain graph. OBNode is an abstract superclass for concrete nodes which might appear in the browser.

iVars:

metaNode - the MetaNode which produced this node!

OBNode subclass: #OBCollectionNode
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Nodes'!

!OBCollectionNode commentStamp: 'cwp 1/7/2005 23:31' prior: 0!
OBCollectionNode is a trivial wrapper for a collection of nodes. It is typically used as an artificial root node for metagraphs that have no natural root.!

!OBCollectionNode class methodsFor: 'as yet unclassified' stamp: 'cwp 3/15/2004 23:17'!
on: aCollection
	^ self new setCollection: aCollection! !

!OBCollectionNode methodsFor: 'navigating' stamp: 'cwp 4/15/2007 23:20'!
addChild: aNode
	(collection includes: aNode)
		ifFalse: [collection _ collection copyWith: aNode]! !

!OBCollectionNode methodsFor: 'navigating' stamp: 'cwp 3/15/2004 23:19'!
children
	^ collection! !

!OBCollectionNode methodsFor: 'ancestry' stamp: 'cwp 4/3/2007 22:34'!
isAncestorOf: aNode
	^ collection anySatisfy: [:ea | ea = aNode or: [ea isAncestorOf: aNode]]! !

!OBCollectionNode methodsFor: 'displaying' stamp: 'cwp 3/15/2004 23:19'!
name
	^ collection species name! !

!OBCollectionNode methodsFor: 'initalizing' stamp: 'cwp 3/15/2004 23:18'!
setCollection: aCollection
	collection _ aCollection! !

!OBNode class methodsFor: 'as yet unclassified' stamp: 'cwp 2/5/2004 20:25'!
actionsForParent: aNode
	^ #()! !

!OBNode class methodsFor: 'as yet unclassified' stamp: 'cwp 12/18/2003 22:11'!
fromAssociation: anAssociationNode
	^ self on: anAssociationNode value! !

!OBNode methodsFor: 'comparing' stamp: 'cwp 1/31/2004 14:57'!
= other
	^ self class = other class and: [self name = other name]! !

!OBNode methodsFor: 'public' stamp: 'cwp 2/29/2004 13:49'!
accept: aText notifying: aController
	^ self text: aText! !

!OBNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:28'!
acceptDroppedNode: aNode
 	^ aNode perform: self dropSelector with: self
! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 00:44'!
ancestrySelector
	^ #isDescendantOf: ! !

!OBNode methodsFor: 'updating' stamp: 'cwp 6/4/2006 12:18'!
announce: anObject
	^ OBAnnouncer current ifNotNilDo: [:announcer | announcer announce: anObject]! !

!OBNode methodsFor: 'updating' stamp: 'cwp 3/13/2007 00:18'!
announceChangedWith: anObject
	anObject announce: (OBNodeChanged node: self)! !

!OBNode methodsFor: 'updating' stamp: 'cwp 3/13/2007 00:21'!
announceDeletionWith: anObject
	anObject announce: (OBNodeDeleted node: self)
! !

!OBNode methodsFor: 'updating' stamp: 'cwp 3/13/2007 00:34'!
announceSelectionWith: anObject
	(anObject announce: OBAboutToChange)
		isVetoed ifFalse: [anObject announce: (OBSelectingNode node: self)]! !

!OBNode methodsFor: 'drag and drop' stamp: 'avi 2/20/2004 14:00'!
asDraggableMorph
	^(StringMorph contents: self name)
		color: Color white;
		yourself! !

!OBNode methodsFor: 'public' stamp: 'cwp 3/22/2004 20:48'!
definition
	^ self! !

!OBNode methodsFor: 'updating' stamp: 'cwp 6/4/2006 01:06'!
demandSelection
	^ OBAnnouncer current announce: (OBNodeCreated node: self)! !

!OBNode methodsFor: 'public' stamp: 'dvf 8/17/2005 13:57'!
displayName
	"expected to return a decorated text (rather than string) version of name"
	^self name asText! !

!OBNode methodsFor: 'displaying' stamp: 'cwp 12/8/2004 21:36'!
displayString
	^ self perform: metaNode displaySelector! !

!OBNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:29'!
dropSelector
	"Override in subclasses"
	
	^ #dropOnNode: ! !

!OBNode methodsFor: 'comparing' stamp: 'cwp 1/31/2004 14:57'!
hash
	^ self name hash! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 01:47'!
isAncestorOf: aNode
	^ aNode perform: self ancestrySelector with: self! !

!OBNode methodsFor: 'ancestry' stamp: 'cwp 3/3/2004 00:44'!
isDescendantOf: aNode
	^ false! !

!OBNode methodsFor: 'accessing' stamp: 'cwp 3/3/2004 23:24'!
metaNode
	^ metaNode! !

!OBNode methodsFor: 'accessing' stamp: 'cwp 3/3/2004 23:24'!
metaNode: aMetaNode
	metaNode _ aMetaNode! !

!OBNode methodsFor: 'public' stamp: 'cwp 12/10/2003 22:27'!
name
	self subclassResponsibility! !

!OBNode methodsFor: 'utility' stamp: 'cwp 2/7/2004 21:29'!
referenceForMethod: selector ofClass: className
	| classIsMeta symbol |
	classIsMeta _ className endsWith: ' class'.
	symbol _ classIsMeta ifTrue: [(className allButLast: 6) asSymbol] ifFalse: [className].
	^ MethodReference new 
		setClassSymbol: symbol
		classIsMeta: classIsMeta
		methodSymbol: selector
		stringVersion: symbol, '>>', selector! !

!OBNode methodsFor: 'updating' stamp: 'cwp 6/4/2006 12:15'!
signalChanged
	self announce: (OBNodeChanged node: self)! !

!OBNode methodsFor: 'updating' stamp: 'cwp 6/4/2006 12:15'!
signalChildrenChanged
	self announce: (OBChildrenChanged node: self)! !

!OBNode methodsFor: 'updating' stamp: 'cwp 6/4/2006 12:15'!
signalDeletion
	self announce: (OBNodeDeleted node: self)
! !

!OBNode methodsFor: 'updating' stamp: 'cwp 6/4/2006 13:59'!
signalSelection
	(self announce: OBAboutToChange)
		isVetoed ifFalse: [self announce: (OBSelectingNode node: self)]! !

!OBNode methodsFor: 'public' stamp: 'cwp 12/7/2003 19:27'!
text
	^ ''! !

!OBNode methodsFor: 'public' stamp: 'cwp 3/14/2004 14:09'!
text: aText
	^ false! !

!OBNode methodsFor: 'public' stamp: 'cwp 2/29/2004 18:30'!
textSelection
	^ 1 to: 0! !

!OBNode methodsFor: 'displaying' stamp: 'cwp 8/28/2004 00:14'!
title
	^ nil! !

!OBNode methodsFor: 'drag and drop' stamp: 'cwp 3/2/2004 21:28'!
wantsDroppedNode: aNode
	^ aNode respondsTo: self dropSelector! !

Object subclass: #OBPanel
	instanceVariableNames: 'browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Kernel'!

!OBPanel commentStamp: 'cwp 1/7/2005 23:23' prior: 0!
A panel is an object that manages part of the browser's window. It provides a protocol for receiving notifications when the current domain node changes, and reacts to these changes.!

OBPanel subclass: #OBColumnPanel
	instanceVariableNames: 'root columns minPanes maxPanes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBColumnPanel commentStamp: 'cwp 12/6/2004 23:50' prior: 0!
OBColumnPanel handles navigation around the nodes of the object graph. It maintains a list of columns, which track the user's path through the node tree. As nodes are selected, additional columns are added to the list, which appear as panes on the right of the panel. 

iVars:

columns	- A collection of OBColumns, each of which manages a single pane in the scroller.
minPanes - The minimum number of panes that should ever be visible.
maxPanes - The maximum number of panes that should ever be visible.
!

!OBColumnPanel class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 16:56'!
minPanes: min maxPanes: max
	^ self basicNew setMinPanes: min maxPanes: max! !

!OBColumnPanel class methodsFor: 'instance creation' stamp: 'cwp 12/5/2004 16:56'!
new
	^ self minPanes: 1 maxPanes: 1! !

!OBColumnPanel methodsFor: 'building' stamp: 'cwp 3/16/2007 23:09'!
buildOn: aBuilder
	^ aBuilder 
		columnPanel: self 
		with: [columns do: [:ea | ea buildOn: aBuilder]]! !

!OBColumnPanel methodsFor: 'constructing' stamp: 'cwp 11/17/2004 23:24'!
buildScroller
	^ (OBPaneScroller withModel: self)
		name: 'scroller';
		yourself! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 9/25/2005 23:58'!
clearAfter: aColumn 
	| start |
	start _ (columns indexOf: aColumn) + 1.
	start to: columns size do: [:i | (columns at: i) clear]! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 2/28/2006 18:12'!
columnAfter: aColumn 
	^ [self columns after: aColumn]
		on: Error
		do: [:err | nil]
! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 2/28/2006 19:07'!
columnBefore: aColumn 
	^ self columnBefore: aColumn ifAbsent: [nil]

! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 2/28/2006 19:07'!
columnBefore: aColumn ifAbsent: aBlock
	^ [self columns before: aColumn]
		on: Error
		do: [:err | aBlock value]


! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:46'!
columns
	^columns! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:46'!
columns: anObject
	columns := anObject! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 11/16/2004 22:22'!
currentColumn
	^self columns reversed detect: [:ea | ea hasSelection] ifNone: []! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 22:22'!
currentNode
	^ self currentColumn ifNotNilDo: [:column | column selectedNode]! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 22:23'!
currentOrRootNode
	^(self columns reversed detect: [:ea | ea hasSelection]
		ifNone: [^self columns first parent]) selectedNode! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/26/2004 21:55'!
defaultBackgroundColor
	^ browser defaultBackgroundColor! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 11/16/2004 23:01'!
emptyColumn
	^ OBColumn inPanel: self! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 6/5/2006 23:10'!
hasSelection
	^ false! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 6/9/2006 23:18'!
hopTo: aNode 
	| column |
	column := self columns last.
	[column refreshAndSignal: false; includesNode: aNode]
		whileFalse: [column := self 
						columnBefore: column 
						ifAbsent: [^ self jumpTo: aNode]].
	column select: aNode! !

!OBColumnPanel methodsFor: 'testing' stamp: 'cwp 11/20/2004 21:10'!
isNavigation
	^ true! !

!OBColumnPanel methodsFor: 'constructing' stamp: 'cwp 2/25/2007 20:40'!
isVariableHeight
	^ true! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 9/27/2006 22:32'!
jumpTo: aNode 
	| column |
	column _ self selectAncestorsOf: aNode.	
	self announcer announce: (OBSelectionChanged column: column)! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 11/16/2004 23:34'!
jumpToRoot
	^ self columns first selection: 0! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 23:51'!
labelString
	| label |
	self columns reversed do: 
		[:ea | 
		label := ea selectedNode ifNotNilDo: [:node | node title].
		label ifNotNil: [^ label]].
	^ nil
! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 23:29'!
maxPanes
	^maxPanes! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 22:54'!
minPanes
	^ minPanes! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 6/5/2006 23:52'!
nodeDeleted: ann
	ann node = self root 
		ifTrue: 
			[self columns first clear.
			self announcer announce: (OBSelectionChanged column: self)]! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/23/2004 00:52'!
okToReclaimPane
	columns size > minPanes ifFalse: [^ false].
	^ columns last isEmpty or: [(columns at: columns size - 1) shouldBeLast].
! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 2/28/2006 09:59'!
parentNodeForColumn: aColumn
	^ [(columns before: aColumn) selectedNode]
		on: Error
		do: [:err | ^ root]! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 6/4/2006 00:39'!
popColumn
	self announcer unsubscribe: self columns removeLast.
! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 3/25/2007 00:24'!
pushColumn: aColumn 
	self columns addLast: aColumn.
	self changed: #columns.
! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 9/18/2005 11:49'!
reclaimPanes
	| old |
	old := columns size.
	[self okToReclaimPane] whileTrue: [self popColumn].
	^ old - columns size! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 9/24/2005 22:47'!
root
	^ root! !

!OBColumnPanel methodsFor: 'navigating' stamp: 'cwp 9/27/2006 22:44'!
selectAncestorsOf: aNode 
	| ancestor |

	self columns do: 
		[:column |
		ancestor := column selectAncestorOf: aNode.
		(ancestor = aNode or: [ancestor isNil])
			ifTrue: [^ column]]
! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 6/4/2006 01:03'!
selectNode: announcement
	^ self hopTo: announcement node! !

!OBColumnPanel methodsFor: 'accessing columns' stamp: 'cwp 6/8/2006 01:35'!
selected: aColumn 
	aColumn next ifNotNilDo: 
		[:next | 
		next parent: aColumn selectedNode].! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'cwp 3/11/2007 21:08'!
selectedNode
	^ self currentNode! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 6/5/2006 01:33'!
selectionChanged: ann
	ann column hasSelection
		ifTrue: [self selected: ann column]
		ifFalse: [self clearAfter: ann column]
! !

!OBColumnPanel methodsFor: 'accessing' stamp: 'dvf 9/5/2005 18:08'!
selectionPath
	^(self columns 
		collect: [:e | e parent] 
		thenSelect: [:e | e notNil]) allButFirst! !

!OBColumnPanel methodsFor: 'initializing' stamp: 'cwp 9/24/2005 22:45'!
setMetaNode: aMetaNode node: aNode 
	root _ aNode.
	root metaNode: aMetaNode.
	self pushColumn: (aMetaNode columnInPanel: self node: root).
	minPanes - self columns size 
		timesRepeat: [self pushColumn: self emptyColumn]! !

!OBColumnPanel methodsFor: 'initializing' stamp: 'cwp 12/5/2004 16:54'!
setMinPanes: min maxPanes: max
	columns _ OrderedCollection new.
	minPanes _ min.
	maxPanes _ max! !

!OBColumnPanel methodsFor: 'callbacks' stamp: 'cwp 11/18/2004 00:00'!
sizing
	^ (columns size max: minPanes) min: maxPanes! !

!OBColumnPanel methodsFor: 'updating' stamp: 'cwp 6/5/2006 01:32'!
subscribe
	self announcer
		observe: OBSelectingNode send: #selectNode: to: self;
		observe: OBSelectionChanged send: #selectionChanged: to: self;
		observe: OBNodeCreated send: #selectNode: to: self;
		observe: OBNodeDeleted send: #nodeDeleted: to: self.! !

!OBColumnPanel methodsFor: 'building' stamp: 'cwp 3/18/2007 20:36'!
vResizing
	^ #spaceFill! !

OBPanel subclass: #OBFixedButtonPanel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBFixedButtonPanel commentStamp: 'cwp 1/7/2005 23:35' prior: 0!
OBFixedButtonPanel displays a horizontal row of buttons. In contrast to OBVarButtonPanel, the buttons do not change as nodes are selected in the navigation panel; instead they are enabled and disabled according to whether the actions they represent are applicable to the selected node.!

!OBFixedButtonPanel class methodsFor: 'as yet unclassified' stamp: 'cwp 2/28/2006 10:41'!
new
	^ self basicNew initialize! !

!OBFixedButtonPanel methodsFor: 'building' stamp: 'cwp 3/18/2007 22:13'!
buildOn: aBuilder
	^ aBuilder
		fixedButtonPanel: self
		with: []! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 3/11/2007 20:12'!
color
	^ browser defaultBackgroundColor ! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 3/11/2007 21:15'!
commands
	| commands node |
	node _ self currentNode.
	node ifNil: [^ #()].
	commands _ (self announce: OBNodeCommandScan) commandsOn: node for: self.
	^ (commands select: [:ea | ea wantsButton]) asSortedCollection: [:a :b | a label <= b label]! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 3/11/2007 21:15'!
currentNode
	^ browser currentOrRootNode ! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 3/11/2007 21:14'!
isSelected: aNode
	^ true! !

!OBFixedButtonPanel methodsFor: 'constructing' stamp: 'cwp 2/25/2007 20:40'!
isVariableHeight
	^ false! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 3/11/2007 21:14'!
selectionChanged: ann
	self changed: #commands! !

!OBFixedButtonPanel methodsFor: 'callbacks' stamp: 'cwp 6/3/2006 22:14'!
subscribe
	self announcer
		observe: OBSelectionChanged
		send: #selectionChanged:
		to: self! !

!OBFixedButtonPanel methodsFor: 'building' stamp: 'cwp 3/18/2007 22:16'!
vResizing
	^ #rigid! !

!OBPanel class methodsFor: 'instance creation' stamp: 'cwp 8/31/2004 10:46'!
inBrowser: aBrowser
	^ self new browser: aBrowser! !

!OBPanel methodsFor: 'updating' stamp: 'cwp 10/13/2006 09:29'!
announce: aClass 
	^ self browser announce: aClass! !

!OBPanel methodsFor: 'accessing' stamp: 'cwp 4/17/2006 15:52'!
announcer
	^ browser announcer! !

!OBPanel methodsFor: 'accessing' stamp: 'cwp 11/16/2004 21:58'!
browser
	^ browser! !

!OBPanel methodsFor: 'accessing' stamp: 'cwp 4/17/2006 22:17'!
browser: aBrowser
	browser _ aBrowser.
	self subscribe! !

!OBPanel methodsFor: 'testing' stamp: 'cwp 11/20/2004 21:09'!
isNavigation
	^ false! !

!OBPanel methodsFor: 'updating' stamp: 'cwp 4/17/2006 19:36'!
subscribe! !

OBPanel subclass: #OBTextPanel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

OBTextPanel subclass: #OBDefinitionPanel
	instanceVariableNames: 'definition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Panels'!

!OBDefinitionPanel commentStamp: 'cwp 1/7/2005 23:35' prior: 0!
OBDefinition serves as the model for the text pane of a typical browser. It's main responsibility is to act as a relay between a PluggableTextMorph and a Definition supplied by the currently selected node.

iVars:

browser - The browser of which this panel is a part.
!

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 4/18/2006 21:31'!
aboutToChange: ann
	| ok |
	self canDiscardEdits ifTrue: [^ true].
	ok _ OBConfirmationRequest
			prompt: 'Changes have not been saved.
Is it OK to discard those changes?'
			confirm: 'Discard changes'.
	ok
		ifTrue: [self changed: #clearUserEdits]
		ifFalse: [ann veto]! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 3/3/2007 23:11'!
aboutToChangeSilently: ann
	self canDiscardEdits ifFalse: [ann veto]! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 6/4/2006 00:41'!
accept: aText notifying: aController
	^ self 
		withDefinitionDo: [:def | [def accept: aText notifying: aController]
									on: OBAnnouncerRequest
									do: [:notification | notification resume: self announcer]]
		ifNil: [true]! !

!OBDefinitionPanel methodsFor: 'building' stamp: 'cwp 3/16/2007 23:09'!
buildOn: aBuilder
	^ aBuilder definitionPanel: self with: []! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'cwp 9/17/2004 00:56'!
definition: aDefinition
	definition _ aDefinition.
	self changed: #text! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 6/4/2006 11:31'!
definitionChanged: ann
	definition _ ann definition.
	self changed: #text! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:23'!
doItContext
	^ self 
		withDefinitionDo: [:def | (def respondsTo: #doItContext) ifTrue: [def doItContext]]
		ifNil: [nil]! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:23'!
doItReceiver
	^ self 
		withDefinitionDo: [:def | (def respondsTo: #doItReceiver) ifTrue: [def doItReceiver]]
		ifNil: [nil]! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'cwp 4/19/2007 21:56'!
environment
	^ self selectedClass environment! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'cwp 3/22/2004 20:06'!
getDefinition
	^ browser currentNode ifNotNilDo: [:node | node definition]! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 6/3/2006 21:55'!
refresh: announcement
	| oldDefinition |
	definition ifNil: [^ self].
	oldDefinition _ definition.
	definition _ self getDefinition.
	definition text = oldDefinition text ifTrue: [^ self].
	self canDiscardEdits
			ifTrue: [self changed: #text]
			ifFalse: [self changed: #codeChangedElsewhere]! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/23/2004 00:22'!
selectedClass
	^ self 
		withDefinitionDo: [:def | (def respondsTo: #selectedClass) ifTrue: [def selectedClass]] 
		ifNil: [nil]! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 21:15'!
selection
	^ self withDefinitionDo: [:def | def textSelection] ifNil: [1 to: 0]! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 4/24/2006 16:06'!
selectionChanged: ann
	self definition: self getDefinition! !

!OBDefinitionPanel methodsFor: 'updating' stamp: 'cwp 3/3/2007 22:49'!
subscribe
	self announcer 
		observe: OBAboutToChange
		send: #aboutToChange:
		to: self;
		
		observe: OBAboutToChangeSilently
		send: #aboutToChangeSilently:
		to: self;
		
		observe: OBSelectionChanged
		send: #selectionChanged:
		to: self;
		
		observe: OBRefreshRequired
		send: #refresh:
		to: self;
				
		observe: OBDefinitionChanged
		send: #definitionChanged: 
		to: self.! !

!OBDefinitionPanel methodsFor: 'callbacks' stamp: 'cwp 3/22/2004 20:42'!
text
	^ self withDefinitionDo: [:def | def text] ifNil: ['']! !

!OBDefinitionPanel methodsFor: 'accessing' stamp: 'cwp 3/22/2004 20:41'!
withDefinitionDo: workBlock ifNil: nilBlock
	definition ifNil: [ ^ nilBlock value].
	^ workBlock value: definition! !

!OBTextPanel methodsFor: 'user interface' stamp: 'cwp 4/19/2007 21:35'!
addEditingItemsToMenu: aMenu shifted: aBoolean
	| items |
	items _  aBoolean 
		ifTrue: [self shiftedYellowButtonMenu] 
		ifFalse: [self yellowButtonMenu].
	items do: [:ea |
			  ea = #-
				ifFalse: [aMenu add: ea first action: ea second]
				ifTrue: [aMenu addLine]].
! !

!OBTextPanel methodsFor: 'user interface' stamp: 'cwp 4/19/2007 21:35'!
addMoreToMenu: aMenu
	aMenu addLine.
	aMenu
		add: 'more...' translated
		action: #shiftedTextPaneMenuRequest! !

!OBTextPanel methodsFor: 'user interface' stamp: 'cwp 4/19/2007 22:08'!
addTextCommandsToMenu: aMenu selection: aTextSelection
	| scan |
	scan _ self announce: OBTextCommandScan.
	scan
		populateMenu: aMenu
		withNode: aTextSelection
		forRequestor: self.
! !

!OBTextPanel methodsFor: 'callbacks' stamp: 'cwp 4/19/2007 22:08'!
menu: aMenu shifted: aBoolean selection: aTextSelection 
	
	self addEditingItemsToMenu: aMenu shifted: aBoolean.
	self addTextCommandsToMenu: aMenu selection: aTextSelection.
	self addMoreToMenu: aMenu.
	^ aMenu ! !

!OBTextPanel methodsFor: 'callbacks' stamp: 'cwp 4/19/2007 21:32'!
perform: aSelector orSendTo: anObject
	| receiver |
	receiver  _ (self respondsTo: aSelector) ifTrue: [self] ifFalse: [anObject].
	receiver perform: aSelector! !

!OBTextPanel methodsFor: 'user interface' stamp: 'cwp 4/19/2007 21:36'!
shiftedYellowButtonMenu
	^ {
		{'explain' translated.						#explain}.
		#-.
		{'selectors containing it (W)' translated.		#methodNamesContainingIt}.
		{'method strings with it (E)' translated.		#methodStringsContainingit}.
		{'method source with it' translated.			#methodSourceContainingIt}.
		{'class names containing it' translated.		#classNamesContainingIt}.
		{'class comments with it' translated.			#classCommentsContainingIt}.
		{'change sets with it' translated.				#browseChangeSetsWithSelector}.
		#-.
		{'save contents to file...' translated.			#saveContentsInFile}.
		#-.
		{'more...' translated.							#yellowButtonActivity}.
	}
! !

!OBTextPanel methodsFor: 'user interface' stamp: 'cwp 4/19/2007 21:36'!
yellowButtonMenu
	^ {
		{'find...(f)' translated.				#find}.			     
		{'find again (g)' translated.			#findAgain}.		     
		{'set search string (h)' translated.	#setSearchString}.
		#-.	     
		{'do again (j)' translated.			#again}.		     
		{'undo (z)' translated.				#undo}.			     
		#-.	     
		{'copy (c)' translated.				#copySelection}.	     
		{'cut (x)' translated.					#cut}.			     
		{'paste (v)' translated.				#paste}.		     
		{'paste...' translated.					#pasteRecent}.		     
		#-.	     
		{'accept (s)' translated.				#accept}.		     
		{'cancel (l)' translated.				#cancel}.		     
	}
! !

Object subclass: #OBRequestor
	instanceVariableNames: 'node browser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Commands'!

!OBRequestor class methodsFor: 'as yet unclassified' stamp: 'cwp 6/28/2006 00:40'!
for: aBrowser
	^ self new browser: aBrowser! !

!OBRequestor methodsFor: 'as yet unclassified' stamp: 'cwp 6/29/2006 00:12'!
announce: anObject
	^ browser announcer announce: anObject! !

!OBRequestor methodsFor: 'as yet unclassified' stamp: 'cwp 6/28/2006 00:41'!
browser: aBrowser
	browser _ aBrowser! !

!OBRequestor methodsFor: 'as yet unclassified' stamp: 'cwp 6/28/2006 00:43'!
doesNotUnderstand: aMessage
	^ aMessage sendTo: browser! !

!OBRequestor methodsFor: 'as yet unclassified' stamp: 'cwp 6/28/2006 00:43'!
node: aNode
	node _ aNode! !

!OBRequestor methodsFor: 'as yet unclassified' stamp: 'cwp 6/28/2006 00:38'!
requestNode
	^ node! !

!OBRequestor methodsFor: 'as yet unclassified' stamp: 'cwp 6/29/2006 00:12'!
select: aNode
	(self announce: OBAboutToChange)
		isVetoed ifFalse: [self announce: (OBSelectingNode node: aNode)]! !

Object subclass: #OBTextSelection
	instanceVariableNames: 'selection text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniBrowser-Utilities'!

!OBTextSelection class methodsFor: 'instance creation' stamp: 'cwp 10/13/2006 16:52'!
on: anOBPluggableTextMorph 
	^ self new! !

!OBTextSelection class methodsFor: 'instance creation' stamp: 'cwp 10/14/2006 20:10'!
on: anInterval inText: aString
	^ self new setSelection: anInterval inText: aString! !

!OBTextSelection class methodsFor: 'instance creation' stamp: 'cwp 10/14/2006 20:59'!
onAllOf: aString 
	^ self on: (1 to: aString size) inText: aString! !

!OBTextSelection methodsFor: 'as yet unclassified' stamp: 'cwp 10/14/2006 20:15'!
fullText
	^ text! !

!OBTextSelection methodsFor: 'as yet unclassified' stamp: 'cwp 10/14/2006 20:44'!
hasSelector
	^ self selector notNil! !

!OBTextSelection methodsFor: 'as yet unclassified' stamp: 'cwp 10/14/2006 21:31'!
selector
	^ self text asString findSelector! !

!OBTextSelection methodsFor: 'as yet unclassified' stamp: 'cwp 10/14/2006 20:12'!
setSelection: anInterval inText: aString
	selection _ anInterval.
	text _ aString.! !

!OBTextSelection methodsFor: 'as yet unclassified' stamp: 'cwp 10/14/2006 20:14'!
text
	^ text atAll: selection! !

!Object methodsFor: '*omnibrowser-converting' stamp: 'cwp 4/17/2006 12:16'!
asAnnouncement
	^ self! !

!PluggableButtonMorph methodsFor: '*omnibrowser' stamp: 'cwp 2/24/2004 19:07'!
isOn
	^ self color = onColor! !
